/*

  silcsocketstream.h

  Author: Pekka Riikonen <priikone@silcnet.org>

  Copyright (C) 2005 Pekka Riikonen

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; version 2 of the License.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

*/

/****h* silcutil/SILC Socket Stream Interface
 *
 * DESCRIPTION
 *
 * Implementation of SILC Socket Stream.  SILC Socket Stream can be used
 * read data from and write data to a socket connection.
 *
 ***/

#ifndef SILCSOCKETSTREAM_H
#define SILCSOCKETSTREAM_H

/****s* silcutil/SilcSocketStreamAPI/SilcSocketStream
 *
 * NAME
 *
 *    typedef struct SilcSocketStreamStruct *SilcSocketStream;
 *
 * DESCRIPTION
 *
 *    The SILC Socket Stream context.  This context is allocated using
 *    silc_socket_stream_create function.  All data reading and writing,
 *    and other stream handling is done using the generic SilcStream API,
 *    eg. silc_stream_read and silc_stream_write.
 *
 ***/
typedef struct SilcSocketStreamStruct *SilcSocketStream;

/****d* silcutil/SilcSocketStreamAPI/SilcSocketStreamStatus
 *
 * NAME
 *
 *    typedef enum { ... } SilcStreamStatus;
 *
 * DESCRIPTION
 *
 *    Socket Stream status.  This status is returned into the
 *    SilcSocketStreamCallback function after the socket stream is
 *    created.
 *
 * SOURCE
 */
typedef enum {
  SILC_SOCKET_OK,		/* Normal status */
  SILC_SOCKET_UNKNOWN_IP,	/* Remote does not have IP address */
  SILC_SOCKET_UNKNOWN_HOST,	/* Remote does not have host name */
  SILC_SOCKET_NO_MEMORY,	/* System out of memory */
  SILC_SOCKET_ERROR,		/* Unknown error */
} SilcSocketStreamStatus;
/***/

/****s* silcutil/SilcSocketStreamAPI/SilcSocketQos
 *
 * NAME
 *
 *    typedef struct SilcSocketQosStruct *SilcSocketQos;
 *
 * DESCRIPTION
 *
 *    This structure is "Quality of Service" structure for the socket
 *    stream and is set with silc_socket_stream_set_qos function for the
 *    socket stream.
 *
 ***/
typedef struct SilcSocketQosStruct {
  SilcUInt16 read_limit_bytes;	    /* Max read bytes */
  SilcUInt16 read_rate;		    /* Max read rate/second */
  SilcUInt16 limit_sec;		    /* Limit seconds */
  SilcUInt32 limit_usec;	    /* Limit microseconds */
  struct timeval next_limit;
  unsigned int cur_rate : 31;
  unsigned int applied  : 1;
  SilcUInt32 data_len;
  SilcSocketStream sock;
} *SilcSocketQos;

/****f* silcutil/SilcSocketStreamAPI/SilcSocketStreamCallback
 *
 * SYNOPSIS
 *
 *    typedef void (*SilcSocketStreamCallback)(SilcSocketStreamStatus status,
 *                                             SilcStream stream,
 *                                             void *context);
 *
 * DESCRIPTION
 *
 *    Callback function of this type is called after the socket stream
 *    creation is completed.  If the `stream' is NULL the socket stream could
 *    not be created of the socket connection is not otherwise allowed.  The
 *    `status' will indicate the error status.  The `stream' is socket stream
 *    representing the socket connection and silc_socket_stream_* functions
 *    can be used to access the stream.  All other silc_stream_* functions
 *    can also be used to read data, send data, and otherwise handle the
 *    stream.
 *
 ***/
typedef void (*SilcSocketStreamCallback)(SilcSocketStreamStatus status,
					 SilcStream stream, void *context);

/****f* silcutil/SilcSocketStreamAPI/silc_socket_stream_create
 *
 * SYNOPSIS
 *
 *    SilcAsyncOperation
 *    silc_socket_stream_create(int sock, bool require_fqdn,
 *                              SilcSchedule schedule,
 *                              SilcSocketStreamCallback callback,
 *                              void *context);
 *
 * DESCRIPTION
 *
 *    Creates new socket stream of the socket connection indicated by `sock'.
 *    The stream can be destroyed by calling the silc_stream_destroy.  Data
 *    can be sent and received from the stream by calling silc_stream_write
 *    and silc_stream_read.  The creation process is asynchronous since
 *    socket connection information, such as hostname and IP address are
 *    resolved, so SilcAsyncOperation is returned which can be used to cancel
 *    the creationg process.  The `callback' will be called to return the
 *    created socket stream
 *
 ***/
SilcAsyncOperation
silc_socket_stream_create(int sock, bool require_fqdn,
			  SilcSchedule schedule,
			  SilcSocketStreamCallback callback,
			  void *context);

/****f* silcutil/SilcSocketStreamAPI/silc_socket_stream_get_info
 *
 * SYNOPSIS
 *
 *    bool
 *    silc_socket_stream_get_info(SilcSocketStream stream,
 *                                const char **hostname,
 *                                const char **ip, SilcUInt16 *port);
 *
 * DESCRIPTION
 *
 *    Returns socket stream information such as the hostname, IP address and
 *    the port of the remote socket connection.  Return FALSE if these
 *    informations are not available.
 *
 ***/
bool silc_socket_stream_get_info(SilcSocketStream stream,
				 const char **hostname,
				 const char **ip, SilcUInt16 *port);

/****f* silcutil/SilcSocketStreamAPI/silc_socket_stream_get_error
 *
 * SYNOPSIS
 *
 *    int silc_socket_stream_get_error(SilcSocketStream stream);
 *
 * DESCRIPTION
 *
 *    If error occurred during socket stream operations, this function
 *    can be used to retrieve the error number that occurred.
 *
 ***/
int silc_socket_stream_get_error(SilcSocketStream stream);

/****f* silcutil/SilcSocketStreamAPI/silc_socket_stream_set_qos
 *
 * SYNOPSIS
 *
 *    bool silc_socket_stream_set_qos(SilcSocketStream stream,
 *                                    SilcUInt32 read_rate,
 *                                    SilcUInt32 read_limit_bytes,
 *                                    SilcUInt32 limit_sec,
 *                                    SilcUInt32 limit_usec)
 *
 * DESCRIPTION
 *
 *    Sets a "Quality of Service" settings for socket stream `stream'.
 *    The `read_rate' specifies the maximum read operations per second.
 *    If more read operations are executed the limit will be applied for
 *    the reading.  The `read_limit_bytes' specifies the maximum data
 *    that is read.  It is guaranteed that silc_stream_read  never returns
 *    more that `read_limit_bytes' of data.  If more is read the limit
 *    will be applied for the reading.  The `limit_sec' and `limit_usec'
 *    specifies the limit that is applied if `read_rate' and/or
 *    `read_limit_bytes' is reached.  If all arguments except `stream'
 *    are zero this resets the QoS from the socket stream, all QoS for
 *    this socket stream that may be pending will be cancelled.
 *
 ***/
bool silc_socket_stream_set_qos(SilcSocketStream stream,
				SilcUInt32 read_rate,
				SilcUInt32 read_limit_bytes,
				SilcUInt32 limit_sec,
				SilcUInt32 limit_usec);

#endif /* SILCSOCKETSTREAM_H */
